from helpers import is_color, color_string_to_rgb


VENDOR_ID = "1038"
PRODUCT_ID = "1702"


class Rival100:

    """Handle the Rival 100 gaming mouse."""

    EFFECT_STATIC = 0x01
    EFFECT_BREATH = 0x03

    BTN_ACTION_DEFAULT = 0x00
    BTN_ACTION_OS = 0x01

    def __init__(self, device_path):
        self._device = None
        self._device_path = device_path
        self._device_open()

    def set_default(self):
        """Set all option to their default values."""
        self.set_sensitivity(1, 1000)
        self.set_sensitivity(2, 2000)
        self.set_polling_rate(1000)
        self.set_color("#00ffff")
        self.set_light_effect(self.EFFECT_STATIC)
        self.set_btn6_action(self.BTN_ACTION_DEFAULT)

    def set_sensitivity(self, preset, value):
        """Defines a sensor sensitivity preset.

        Arguments:
        preset -- the preset to define (0 or 1)
        value -- the sensitivity (250, 500, 1000, 1250, 1500, 1750, 2000, 4000)
        """
        allowed_values = {
            250: 0x08,
            500: 0x07,
            1000: 0x06,
            1250: 0x05,
            1500: 0x04,
            1750: 0x03,
            2000: 0x02,
            4000: 0x01,
        }
        if preset not in (1, 2):
            raise ValueError()
        if value not in allowed_values:
            raise ValueError()
        self._device_write(0x03, preset, allowed_values[value])

    def set_polling_rate(self, rate):
        """Set the polling rate.

        Arguments:
        rate -- the polling rate (125, 250, 500, 1000)
        """
        rates = {
            125: 0x04,
            250: 0x03,
            500: 0x02,
            1000: 0x01,
        }
        if not rate in rates:
            raise ValueError()
        self._device_write(0x04, 0x00, rates[rate])

    def set_color(self, *args):
        """Set the back-light color.

        Arguments:
        *args -- the color (red, green, blue channel or hexadecimal color or color name)

        Examples:
        set_color(255, 0, 0)
        set_color("red")
        set_color("#ff0000")
        set_color("#f00")
        set_color("ff0000)
        set_color("f00")
        """
        color = (0xFF, 0x00, 0x00)
        if len(args) == 3:
            for value in args:
                if type(value) != int or value < 0 or value > 255:
                    raise ValueError()
            color = args
        elif len(args) == 1 and type(args[0]) == str and is_color(args[0]):
            color = color_string_to_rgb(args[0])
        else:
            raise ValueError()
        self._device_write(0x05, 0x00, *color)

    def set_light_effect(self, effect):
        """Set the light effect (static, breath,...).

        Arguments:
        effect -- the effect (1 or self.EFFECT_STATIC, 2, 3 or self.EFFECT_BREATH, 4)
        """
        if effect < 1 or effect > 4:
            raise ValueError()
        self._device_write(0x07, 0x00, effect)

    def set_btn6_action(self, action):
        """Set the action of the button under the wheel (toggle sensitivity
        presets or controlled by the os).

        Arguments:
        action -- the action (self.BTN_ACTION_DEFAULT, self.BTN_ACTION_OS)
        """
        if action not in (self.BTN_ACTION_DEFAULT, self.BTN_ACTION_OS):
            raise ValueError()
        self._device_write(0x0B, action)

    def save(self):
        """Save the current configuration to the mouse internal memory."""
        self._device_write(0x09, 0x00)

    def _device_open(self):
        """Open the device file"""
        self._device = open(self._device_path, "wb")

    def _device_write(self, *bytes_):
        """Write bytes to the device file.

        Arguments:
        *bytes_ -- bytes to write
        """
        if not self._device:
            return;
        self._device.write(bytearray(bytes_))
        self._device.flush()

    def _device_close(self):
        """Close the device file."""
        if self._device:
            self._device.close()
            self._device = None

    def __del__(self):
        self._device_close()




import sys
from optparse import OptionParser

from helpers import find_hidraw_device_path, usb_device_is_connected
from rival100 import Rival100, VENDOR_ID, PRODUCT_ID
from version import VERSION


parser = OptionParser("Usage: rivalcfg [options]")

parser.add_option("-s", "--sensitivity1",
        help="Set sensitivity preset 1 (allowed values are 250, 500, 1000, 1250, 1500, 1750, 2000 and 4000; default value is 1000)",
        metavar="SENSITIVITY",
        choices=["250", "500", "1000", "1250", "1500", "1750", "2000", "4000"]
        )

parser.add_option("-S", "--sensitivity2",
        help="Set sensitivity preset 2 (allowed values are 250, 500, 1000, 1250, 1500, 1750, 2000 and 4000; default value is 2000)",
        metavar="SENSITIVITY",
        choices=["250", "500", "1000", "1250", "1500", "1750", "2000", "4000"]
        )

parser.add_option("-p", "--polling-rate",
        help="Set polling rate in Hz (allowed values are 125, 250, 500 and 1000; default value is 1000)",
        choices=["125", "250", "500", "1000"]
        )

parser.add_option("-c", "--color",
        help="Set the mouse backlight color (color can be an hexadecimal color or a nammed color; e.g. ff0000, f00, #ff0000, #f00, red,...)"
        )

parser.add_option("-e", "--light-effect",
        help="Set the light effect (allowed values are 1 or static, 2, 3 or breath, 4)",
        choices=["static", "breath", "1", "2", "3", "4"]
        )

parser.add_option("-b", "--btn6-action",
        help="Set the action of the button under the wheel (allowed values are default and os)",
        metavar="ACTION",
        choices=["default", "os", "0", "1"]
        )

parser.add_option("-r", "--reset",
        help="Reset all options to their default values",
        action="store_true"
        )

parser.add_option("-v", "--version",
        help="print the rivalcfg version and exit",
        action="store_true"
        )

options, args = parser.parse_args();


def main():
    if options.version:
        print("rivalcfg %s" % VERSION)
        sys.exit(0)

    if not usb_device_is_connected(VENDOR_ID, PRODUCT_ID):
        print("E: Unable to find any SteelSeries Rival 100 gaming mouse connected to the computer.")
        sys.exit(1)

    device_path = find_hidraw_device_path(VENDOR_ID, PRODUCT_ID)

    if not device_path:
        print("E: The mouse is detected but the control interface is not available.")
        print("\nTry to:")
        print("  * unplug the mouse from the USB port,")
        print("  * wait fiew seconds,")
        print("  * and plug the mouse to the USB port again.")
        sys.exit(1)

    rival = None

    try:
        rival = Rival100(device_path)
    except IOError as e:
        print("E: Cannot open the mouse control interface: %s") % e.strerror
        sys.exit(1)

    if options.reset:
        rival.set_default()
        rival.save()
        sys.exit(0)

    option_setted = False

    if options.sensitivity1 != None:
        option_setted = True
        rival.set_sensitivity(1, int(options.sensitivity1))

    if options.sensitivity2 != None:
        option_setted = True
        rival.set_sensitivity(2, int(options.sensitivity2))

    if options.polling_rate != None:
        option_setted = True
        rival.set_polling_rate(int(options.polling_rate))

    if options.color != None:
        option_setted = True
        try:
            rival.set_color(options.color)
        except ValueError:
            print("E: Invalid color")
            sys.exit(1)

    if options.light_effect != None:
        option_setted = True
        effects = {
            "1": 0x01,
            "2": 0x02,
            "3": 0x03,
            "4": 0x04,
            "static": rival.EFFECT_STATIC,
            "breath": rival.EFFECT_BREATH,
        }
        rival.set_light_effect(effects[options.light_effect])

    if options.btn6_action != None:
        option_setted = True
        actions = {
            "default": rival.BTN_ACTION_DEFAULT,
            "os": rival.BTN_ACTION_OS,
        }
        rival.set_btn6_action(actions[options.btn6_action])

    if option_setted:
        rival.save()
    else:
        print("E: No options provided")
        print("type `rivalcfg -h' to display the options")
        sys.exit(1)

if __name__ == "__main__":
    main()




#!/usr/bin/env python

from setuptools import setup, find_packages
from rivalcfg import VERSION

setup(
    name="rivalcfg",
    version=VERSION,
    description="Configure the SteelSeries Rival 100 gaming mouse",
    long_description=open("README.md").read(),
    url="https://github.com/flozz/rivalcfg",
    license="WTFPL",

    author="Fabien LOISON",
    author_email="http://www.flozz.fr/",

    keywords="steelseries rival rival100 mouse",
    platforms=["Linux"],

    packages=find_packages(),

    install_requires=[
        "pyudev>=0.19.0"
    ],

    entry_points = {
        "console_scripts": [
            "rivalcfg = rivalcfg.__main__:main"
        ]
    },
)

